@@ -6,6 +6,8 @@ import HealpixMap as hmap
 import numpy as np
 from numpy import array, log2, sqrt
 
+import matplotlib.pyplot as plt
+
 class HealpixBase:
     """
     Basic operations related to HEALPix pixelization, for which the map
@@ -40,12 +42,12 @@ class HealpixBase:
         if uniq is not None:
             # MOC map
 
-            self._uniq = uniq
+            self._uniq = array(uniq, dtype = int)
 
             # Scheme and order are implicit
             self._scheme = "NUNIQ"
             
-            self._order = hmap.uniq2order(max(uniq))
+            self._order = hp.nside2order(hmap.uniq2nside(max(uniq)))
             
         else:
             # Single resolution map
@@ -76,47 +78,21 @@ class HealpixBase:
 
     def __eq__(self, other):
 
-        if self.is_moc:
-            return array_equal(self._uniq, other._uniq)
-        else:
-            return (self._order == other._order and 
-                    self._scheme == other._scheme)
-            
-    def uniq(self, pix):
-        """
-        Get the UNIQ value of a pixel (as used in [])
-
-        Args:
-            pix (int or array): Pixel number
-
-        Return:
-            int
-        """
-
-        if self.is_moc:
-
-            return self._uniq[pix]
-
-        else:
-
-            if not self.is_nested:
-                pix = hp.ring2nest(self.nside, pix)
-                
-            return hmap.nest2uniq(self.nside, pix)
+        return self.conformable(other)
             
     @classmethod
-    def adaptive_moc_mesh(cls, split_fun, max_order):
+    def adaptive_moc_mesh(cls, max_nside, split_fun):
         """
         Return a MOC mesh with an adaptive resolution
         determined by an arbitrary function.
 
         Args:
+            max_nside (int): Maximum HEALPix nside to consider
             split_fun (function): This method should return ``True`` if a pixel 
             should be split into pixel of a higher order, and ``False`` otherwise. 
             It takes two integers, ``start`` (inclusive) and ``stop`` (exclusive), 
             which correspond to a single pixel in nested rangeset format for a 
-            map of order ``max_order``.
-            max_order (int): Maximum HEALPix order to consider
+            map of nside ``max_nside``.
 
         Return:
             HealpixBase
@@ -124,6 +100,7 @@ class HealpixBase:
 
         map_uniq = array([], dtype = int)
 
+        max_order = hp.nside2order(max_nside)
         order_list = array(range(max_order+1))
         nside = 2**order_list
         npix_ratio = 4 ** (max_order - order_list)
@@ -170,22 +147,20 @@ class HealpixBase:
         return cls(map_uniq)
 
     @classmethod
-    def moc_from_pixels(cls, pixels, order, nest = False):
+    def moc_from_pixels(cls, nside, pixels, nest = False):
         """
         Return a MOC mesh where a list of pixels are kept at a 
-        given order, and every other pixel is appropiately downsampled.
+        given nside, and every other pixel is appropiately downsampled.
         
         Also see the more generic ``adaptive_moc()`` and ``adaptive_moc_mesh()``.
 
         Args:
+            nside (int): Maximum healpix NSIDE (that is, the NSIDE for the pixel
+                list) 
             pixels (array): Pixels that must be kept at the finest pixelation
-            order (int): Maximum healpix order (that is, the order for the pixel
-                list)
             nest (bool): Whether the pixels are a 'NESTED' or 'RING' scheme
         """
 
-        nside = hp.order2nside(order)
-
         # Always work in nested
         if not nest:
             pixels = array([hp.ring2nest(nside, pix) for pix in pixels])
@@ -200,28 +175,23 @@ class HealpixBase:
             return start_index < stop_index
 
         # Get the grid        
-        return cls.adaptive_moc_mesh(pix_list_range_intersect, 
-                                             order)
+        return cls.adaptive_moc_mesh(nside,
+                                     pix_list_range_intersect)
 
     def conformable(self, other):
         """
         For single-resolution maps, return ``True`` if both maps have the same
-        order and scheme.
+        nside and scheme.
 
         For MOC maps, return `True` if both maps have the same list of UNIQ 
         pixels (including the ordering)
         """
 
-        if self.scheme != other.scheme:
-            return False
-        
         if self.is_moc:
-
             return np.array_equal(self._uniq, other._uniq)
-
         else:
-
-            return self.order == other.order
+            return (self._order == other._order and 
+                    self._scheme == other._scheme)
 
     @property
     def npix(self):
@@ -307,35 +277,47 @@ class HealpixBase:
 
         return self._uniq is not None
 
-    def pix_rangesets(self, order):
+    def pix_rangesets(self, nside):
         """
         Get the equivalent range of `child pixels` in nested scheme for a map 
-        of a higher order
+        of equal or higher nside
 
         Args:
-            order (int): Order of output range sets
+            nside (int): Nside of output range sets
 
         Return:
             recarray: With columns named 'start' (inclusive) and 
                 'stop' (exclusive) 
         """
 
-        if self.scheme == 'NESTED':
+        start, stop = self.pix2range(nside, range(self.npix))
+        
+        return np.rec.fromarrays([start,stop], names = ['start', 'stop'])
 
-            start,stop = hmap.nest2range(self.order, range(self.npix), order)
+    def pix2range(self, nside, pix):
+        """
+        Get the equivalent range of `child pixels` in nested scheme for a map 
+        of equal or higher nside
+        
+        Args:
+            nside (int): Nside of output range sets
+            pix (int or array): Pixel numbers
+            
+        Return:
+            (int or array, int or array): Start pixel (inclusive) and 
+                stop pixel (exclusive) 
+        """
+        
+        if self.is_moc:
 
-        elif self.scheme == 'RING':
-
-            start,stop = hmap.nest2range(self.order,
-                                         hp.ring2nest(self.nside, range(self.npix)),
-                                         order)
+            return hmap.uniq2range(nside, self._uniq[pix])
 
         else:
-            #self.scheme == 'NUNIQ'
 
-            start,stop = hmap.uniq2range(self._uniq, order)
-
-        return np.rec.fromarrays([start,stop], names = ['start', 'stop'])
+            if self.is_ring:
+                pix = hp.ring2nest(self.nside, pix)
+                
+            return hmap.nest2range(self.nside, pix, nside)
 
     def pixarea(self, pix = 0):
         """
@@ -349,11 +331,11 @@ class HealpixBase:
         """
 
         if self.is_moc:
-            order = hmap.uniq2order(self._uniq[pix])
+            nside = hmap.uniq2nside(self._uniq[pix])
         else:
-            order = self.order
+            nside = self.nside
 
-        return 1.047197551196597746 / 4**order
+        return 1.047197551196597746 / nside / nside
         
     def pix2ang(self, pix):
         """
@@ -368,9 +350,7 @@ class HealpixBase:
 
         if self.is_moc:
 
-            pix,order = hmap.uniq2nest(self.pix2uniq(pix))
-
-            nside = hp.order2nside(order)
+            nside, pix = hmap.uniq2nest(self.pix2uniq(pix))
 
             return hp.pix2ang(nside, pix, nest = True)
             
@@ -390,9 +370,7 @@ class HealpixBase:
 
         if self.is_moc:
 
-            pix,order = hmap.uniq2nest(self.pix2uniq(pix))
-
-            nside = hp.order2nside(order)
+            nside, pix = hmap.uniq2nest(self.pix2uniq(pix))
 
             return hp.pix2vec(nside, pix, nest = True)
             
@@ -437,17 +415,17 @@ class HealpixBase:
             
         return pixels
         
-    def pix2uniq(self, ipix):
+    def pix2uniq(self, pix):
         """
         Get the UNIQ representation of a given pixel index.
 
         Args:
-            ipix (int): Pixel number in the current scheme (as used for [])
+            pix (int): Pixel number in the current scheme (as used for [])
         """
 
         if self.is_moc:
 
-            return self._uniq[ipix]
+            return self._uniq[pix]
 
         else:
 
@@ -475,7 +453,7 @@ class HealpixBase:
             # then find pixel number for this order,
             # and then find the rangesets that contain these pixels
 
-            rangesets = self.pix_rangesets(self.order)
+            rangesets = self.pix_rangesets(self.nside)
             
             rs_argsort = np.argsort(rangesets.stop)
             
@@ -485,8 +463,13 @@ class HealpixBase:
 
             opix = rs_argsort[ipix]
 
-            opix[pix == -1] = -1 # Follow healpy convention for null pix
-
+            # Follow healpy convention for null pix
+            if np.ndim(opix) == 0:
+                if pix == -1:
+                    opix = -1
+            else:
+                opix[pix == -1] = -1
+                
             return opix
             
         else:
@@ -545,8 +528,6 @@ class HealpixBase:
         neighbors = hp.get_all_neighbours(self.nside, theta, phi,
                                           nest = self.is_nested)
 
-        print(neighbors)
-        
         if self.is_moc:
             neighbors = self.nest2pix(neighbors)
 
@@ -566,7 +547,7 @@ class HealpixBase:
         if self.is_moc:
 
             # Work in rangesets, and check that there is no gap in between them
-            rs = self.pix_rangesets(self.order)
+            rs = self.pix_rangesets(self.nside)
 
             rs.sort(order = 'start')
 
@@ -600,8 +581,10 @@ class HealpixBase:
 
                 for pix in range(self.npix):
 
-                    nest_pix,order = hmap.uniq2nest(self.uniq(pix))
+                    nside, nest_pix = hmap.uniq2nest(self.pix2uniq(pix))
 
+                    order = hp.nside2order(nside)
+                    
                     pix_per_order[order].append(pix)
                     nest_pix_per_order[order].append(nest_pix)
 
@@ -619,7 +602,7 @@ class HealpixBase:
                     query_pix = np.append(query_pix,
                                           order_pix[query_bool])
 
-                return query_pix
+                return np.sort(query_pix)
                         
             else:
 
@@ -712,9 +695,7 @@ class HealpixBase:
 
             def single_pix_bounds(pix):
 
-                nest_pix, order = hmap.uniq2nest(self._uniq[pix])
-    
-                nside = hp.order2nside(order)
+                nside, nest_pix = hmap.uniq2nest(self._uniq[pix])
     
                 return hp.boundaries(nside, nest_pix, step = step, nest = True)
 
@@ -727,4 +708,250 @@ class HealpixBase:
             return hp.boundaries(self.nside, pix, step, nest = self.is_nested)
 
     
+    def plot_grid(self,
+                  ax = None,
+                  proj = 'moll',
+                  step = 8,
+                  rot=0,
+                  coord='C',
+                  flip='astro',
+                  xsize=800,
+                  ysize=None,
+                  lonra=[-180,180],
+                  latra=[-90,90],
+                  half_sky=False,
+                  reso=1.5,
+                  **kwargs):
+        """
+        Plot the pixel boundaries of a Healpix grid
+
+        Args:
+            m (HealpixBase): Map defining the grid
+            ax (matplotlib.axes.Axes): Axes on where to plot
+            proj (healpy.projector.SphericalProj): Projector to converto 
+                spherical coordinates to plot's axes coodinates
+            step (int): How many points per pixel side
+            rot (float or sequence): Describe the rotation to apply. In the 
+                form (lon, lat, psi) (unit: degrees) : the point at longitude 
+                lon and latitude lat will be at the center. An additional 
+                rotation of angle psi around this direction is applied. If a 
+                scalar, the rotation is performed around zenith
+            coord (str): Either one of ‘G’ (Galactic), ‘E’ (Equatorial) or 
+                ‘C’ (Celestial) to describe the 
+                coordinate system of the map, or a sequence of 2 of these to 
+                rotate the map from the first to the second coordinate system.
+            flip (str): Defines the convention of projection : ‘astro’ 
+                (east towards left, west towards right) or ‘geo’ (east towards 
+                right, west towards left)
+            xsize (int): The horizontal size of the image.
+            ysize (int): The verital size of the image. For carthographic and 
+                gnomonic projections only.
+            lonra (array): Range in longitude (degrees). For carthographic only. 
+            latra (array): Range in latitude (degrees). For carthographic only.
+            half_sky (bool): Plot only one side of the sphere. For orthographic 
+                only
+            reso (float): Resolution (in arcmin). For gnominic projection only.
+            **kwargs: Passed to matplotlib.pyplot.plot()
+        
+        Return:
+            matplotlib.lines.Line2D, healpy,projector: The first return value
+               corresponds to the output ``pyplot.plot()`` for one of the pixels. 
+               The second is the healpy's projector used. This is particularly 
+               useful to add extra elements to the plots, e.g.::
+
+                   plot, proj = m.plot_grid(ax, 'moll')
+                   x,y = proj.ang2xy(np.deg2rad(90), np.deg2rad(45))
+                   ax.text(x, y, "(zenith = 90 deg, azimuth = 45 deg)")
+        """
+
+        # Get appropiate projector
+        projector = self._get_projector(proj, rot, coord, flip, xsize, ysize,
+                                        lonra, latra, half_sky, reso)
+
+        # Create axes is needed
+        if ax is None:
+            plt.figure()        
+            ax = plt.gca()
+            ax.axis('off')
+
+        # Set limits
+        left, right, bottom, top = projector.get_extent()
+        ax.set_xlim(left, right)
+        ax.set_ylim(bottom, top)
+        ax.set_aspect('equal')
+        
+        # Every line should have the same color
+        if 'c' in kwargs:
+            color = kwargs.pop('c')
+        else:
+            color = kwargs.pop('color', 'black')
+
+        # Plot every pixel
+        for pix in range(self.npix):
+
+            # Get appropiate step
+            pix_step = step
+
+            if self.is_moc:
+                pix_nside = hmap.uniq2nside(self.pix2uniq(pix))
+                pix_step *= int(self.nside / pix_nside)
+            
+            # Get boundaries
+            vec = self.boundaries(pix, step = pix_step)
+
+            # Close loop
+            vec = np.append(vec, vec[:,0].reshape(-1,1), axis = 1)
+
+            # Project
+            x,y = projector.vec2xy(vec)
+
+            # Remove discontinuities
+            dx = np.abs(np.diff(x))
+            dy = np.abs(np.diff(y))
+            dist = sqrt(dx*dx + dy*dy)
+            jumps = dist > 3*np.nanmedian(dist)
+            jumps = np.append(jumps, False)
+
+            if any(jumps):
+
+                ijumps = np.nonzero(jumps)[0]+1
+
+                x = np.insert(x, ijumps, np.nan)
+                y = np.insert(y, ijumps, np.nan)
+
+            # Plot
+            plot = ax.plot(x,y, color = color, **kwargs)
+
+        # Add map frame, where discontinuities happend
+        if proj == 'moll':
+
+            # 2:1 ellipse
+            theta = np.linspace(0, 2*np.pi, step * 8 * self.nside)
+            x = 2 * np.cos(theta)
+            y = np.sin(theta)
+            
+            ax.plot(x,y, color = color, **kwargs)
+
+        elif proj == 'cart':
+
+            # 2:1 rectangle
+            x = np.linspace(-180, 180, step * 4 * self.nside)
+            y = np.ones(len(x))
+            ax.plot(x, -90*y, color = color, **kwargs)
+            ax.plot(x,  90*y, color = color, **kwargs)
+
+            y = np.linspace(-90, 90, step * 3 * self.nside)
+            x = np.ones(len(y))
+            ax.plot( 180*x, y, color = color, **kwargs)
+            ax.plot(-180*x, y, color = color, **kwargs)
+
+        elif proj == 'orth':
+
+            # 1 circle per hemisphere
+            theta = np.linspace(0, 2*np.pi, step * 6 * self.nside)
+            x = np.cos(theta)
+            y = np.sin(theta)
+
+            if half_sky:
+                ax.plot(x, y, color = color, **kwargs)
+            else:
+                ax.plot(x-1, y, color = color, **kwargs)
+                ax.plot(x+1, y, color = color, **kwargs)
+            
+        return plot, projector
+            
+    @staticmethod
+    def _get_projector(proj = 'moll',
+                       rot=0,
+                       coord='C',
+                       flip='astro',
+                       xsize=800,
+                       ysize=None,
+                       lonra=[-180,180],
+                       latra=[-90,90],
+                       half_sky=False,
+                       reso=1.5):
+        """
+        Get the appropiate healpy projector
+
+        Args:
+            proj (str): Projections: 'moll' (molltweide), 'cart' (carthographic), 
+                'orth' (orthographics) or 'gnom' (gnomonic)
+            rot (float or sequence): Describe the rotation to apply. In the 
+                form (lon, lat, psi) (unit: degrees) : the point at longitude 
+                lon and latitude lat will be at the center. An additional 
+                rotation of angle psi around this direction is applied. If a 
+                scalar, the rotation is performed around zenith
+            coord (str): Either one of ‘G’ (Galactic), ‘E’ (Equatorial) or 
+                ‘C’ (Celestial) to describe the 
+                coordinate system of the map, or a sequence of 2 of these to 
+                rotate the map from the first to the second coordinate system.
+            flip (str): Defines the convention of projection : ‘astro’ 
+                (east towards left, west towards right) or ‘geo’ (east towards 
+                right, west towards left)
+            xsize (int): The horizontal size of the image.
+            ysize (int): The verital size of the image. For carthographic and 
+                gnomonic projections only.
+            lonra (array): Range in longitude (degrees). For carthographic only. 
+            latra (array): Range in latitude (degrees). For carthographic only.
+            half_sky (bool): Plot only one side of the sphere. For orthographic 
+                only
+            reso (float): Resolution (in arcmin). For gnominic projection only.
+
+
+        Return:
+            healpy.projector
+        """
+
+        proj = proj.lower()
+        
+        if proj == 'moll':            
+            projector = hp.projector.MollweideProj(rot=rot,
+                                                   coord=coord,
+                                                   flipconv=flip,
+                                                   xsize=xsize)
+            
+        elif proj == 'cart':
+            projector = hp.projector.CartesianProj(rot=rot,
+                                                   coord=coord,
+                                                   flipconv=flip,
+                                                   xsize=xsize,
+                                                   ysize=ysize,
+                                                   lonra=lonra,
+                                                   latra=latra)
+        elif proj == 'orth':
+            projector = hp.projector.OrthographicProj(rot=rot,
+                                                      coord=coord,
+                                                      flipconv=flip,
+                                                      xsize=xsize,
+                                                      half_sky=half_sky)
+
+        elif proj == 'gnom':
+            projector = hp.projector.GnomonicProj(rot=rot,
+                                                  coord=coord,
+                                                  flipconv=flip,
+                                                  xsize=xsize,
+                                                  ysize=ysize,
+                                                  reso=reso)
+            
+        else:
+            raise ValueError("Wrong porojection")
+
+        return projector
+        
+    def moc_sort(self):
+        """
+        Sort the uniq pixels composing a MOC map based on its 
+        rangeset representation
+        """
+
+        if not self.is_moc:
+            return
+
+        rs = self.pix_rangesets(self.nside)
+
+        rs_argsort = np.argsort(rs.start)
+
+        self._uniq = self._uniq[rs_argsort]
+        
         
