@@ -41,7 +41,7 @@ class HealpixMap(HealpixBase):
 
 
     Regardless of the underlaying grid, you can operate on maps using 
-    ``*``, ``/``, ``+``, ``-``, ``**``, ``==``, ``abs`` and ``floor``. For binary 
+    ``*``, ``/``, ``+``, ``-``, ``**``, ``==`` and ``abs``. For binary 
     operations the result always corresponds to the finest grid, so there
     is no loss of information. If any of the operands is a MOC, the result is a
     MOC with an appropiate updated grid.. If both operands have the same NSIDE, 
@@ -63,8 +63,10 @@ class HealpixMap(HealpixBase):
     corresponding UNIQ pixel number using ``m.pix2uniq(ipix)``.
 
     Args:
-        data (array): Value for an initialize map. A MOC map is a 2-fields 
-            recarray whose first field is called 'UNIQ'
+        data (array): ``HealpixMap``, for an emty map, or value for an 
+            initialize map. A MOC map is recarray whose first field is called 
+            'UNIQ'. The second field, if it exists, will initialize the contents,
+            otherwise the map will be zero-initialized.
         order (int): Order of HEALPix map.
         nside (int): Alternatively, you can specify the NSIDE parameter.
         scheme (str): Healpix scheme. Either 'RING', 'NESTED' or 'NUNIQ'
@@ -93,7 +95,7 @@ class HealpixMap(HealpixBase):
             data = array(data)
 
             # Single or multi resolution?
-            if (np.size(data.dtype.names) == 2 and
+            if (data.dtype.names is not None and
                 data.dtype.names[0] == "UNIQ"):
                 # Interpret as a multi-order coverage (MOC) map
 
@@ -101,7 +103,11 @@ class HealpixMap(HealpixBase):
 
                 super().__init__(uniq = data.field(0))
 
-                self._data = data.field(1)
+                if len(data.dtype.names) > 1:
+                    self._data = data.field(1)
+                else:
+                    # Empty map
+                    self._data = np.zeros(self.npix, dtype = dtype)
                 
             else:
                 # Single resolution map
@@ -156,8 +162,8 @@ class HealpixMap(HealpixBase):
                 if field is None:
                     field = 1
 
-                uniq = hdu.data.field(uniq_field)
-                contents = hdu.data.field(field)
+                uniq = hdu.data.field(uniq_field).ravel()
+                contents = hdu.data.field(field).ravel()
 
                 data = np.rec.fromarrays([uniq, contents],
                                  names = ['UNIQ', 'CONTENTS'])
@@ -170,7 +176,7 @@ class HealpixMap(HealpixBase):
                 if field is None:
                     field = 0
 
-                m = cls(hdu.data.field(field), scheme=scheme)
+                m = cls(hdu.data.field(field).ravel(), scheme=scheme)
 
         return m
 
@@ -270,15 +276,16 @@ class HealpixMap(HealpixBase):
         
     
     @classmethod
-    def adaptive_moc_map(cls, pix_fun, combine_fun, max_order, density = False):
+    def adaptive_moc_map(cls, max_nside, pix_fun, combine_fun, density = False):
         """
         Discretize a function into an adaptive multi-order map.
 
-        This method evaluates ``pix_fun`` for all pixels in a map of order
-        ``max_order`` and then, guided by ``combine_fun``, recursively groups 
+        This method evaluates ``pix_fun`` for all pixels in a map of nside
+        ``max_nside`` and then, guided by ``combine_fun``, recursively groups 
         together sets of 4 pixels into a single parent pixel of a lower order.
 
         Args:
+            max_nside (int): The function will be initially sampled at this nside
             pix_fun (function): Function to be discretize. It shall take two 
                 arguments: ``nside`` (``int``) and ``pix`` (``int`` or ``array``),
                 and return the value of the function being discretized a for the
@@ -292,7 +299,6 @@ class HealpixMap(HealpixBase):
                 All pixels correspond to the same ``nside``. The function returns
                 ``False`` if these four pixels should remain split, and `True`
                 if they should be combined into a lower order.
-            max_order (int): The function will be initially sampled at this order
             density (bool): If ``True``, the value for pixels at lower order will
                 obtaine by evaluating ``pix_fun`` again, otherwise it will be the
                 sum of the values of the child pixels (i.e. the map is
@@ -303,6 +309,7 @@ class HealpixMap(HealpixBase):
         """
 
         # Cache multiple values for each order
+        max_order = hp.nside2order(max_nside)
         order_list = array(range(max_order, -1, -1))
         nside_list = 2**order_list
         uniq_shift_list = 4 * nside_list * nside_list
@@ -326,7 +333,7 @@ class HealpixMap(HealpixBase):
         nbuffer = np.zeros((max_order+1), dtype=int)
 
         # Loop through all pixels at max_order, 4 at a time
-        for pix in range(0, hmap.order2npix(max_order), 4):
+        for pix in range(0, hp.nside2npix(max_nside), 4):
 
             # Initial evaluation at finer pixelation
             next_pix = pix + 4
@@ -393,19 +400,19 @@ class HealpixMap(HealpixBase):
         return cls(map_data, density = density)
 
     @classmethod
-    def adaptive_moc_mesh(cls, split_fun, max_order, density = False,
+    def adaptive_moc_mesh(cls, max_nside, split_fun, density = False,
                       dtype = None):
         """
         Return a zero-initialized MOC map, with an adaptive resolution
         determined by an arbitrary function.
 
         Args:
+            max_nside (int): Maximum HEALPix nside to consider
             split_fun (function): This method should return ``True`` if a pixel 
             should be split into pixel of a higher order, and ``False`` otherwise. 
             It takes two integers, ``start`` (inclusive) and ``stop`` (exclusive), 
             which correspond to a single pixel in nested rangeset format for a 
-            map of order ``max_order``.
-            max_order (int): Maximum HEALPix order to consider
+            map of nside ``max_nside``.
             density (bool): Will be pass to HealpixMap initialization.
             dtype (dtype): Data type
 
@@ -413,7 +420,7 @@ class HealpixMap(HealpixBase):
             HealpixMap
         """
         
-        base = HealpixBase.adaptive_moc_mesh(split_fun, max_order)
+        base = HealpixBase.adaptive_moc_mesh(max_nside, split_fun)
 
         data = np.rec.fromarrays([base._uniq, np.zeros(base.npix, dtype = dtype)],
                                  names = ["UNIQ", "CONTENTS"])
@@ -421,24 +428,24 @@ class HealpixMap(HealpixBase):
         return cls(data, density = density)
         
     @classmethod
-    def moc_from_pixels(cls, pixels, order, nest = False, density = False,
+    def moc_from_pixels(cls, nside, pixels, nest = False, density = False,
                         dtype = None):
         """
         Return a zero-initialize MOC map where a list of pixels are kept at a 
-        given order, and every other pixel is appropiately downsampled.
+        given nside, and every other pixel is appropiately downsampled.
         
         Also see the more generic ``adaptive_moc_map()`` and ``adaptive_moc_mesh()``.
 
         Args:
+            nside (int): Maximum healpix NSIDE (that is, the NSIDE for the pixel order
+                list) 
             pixels (array): Pixels that must be kept at the finest pixelation
-            order (int): Maximum healpix order (that is, the order for the pixel
-                list)
             nest (bool): Whether the pixels are a 'NESTED' or 'RING' scheme
             density (bool): Wheather the map is density-like or histogram-like
             dtype: Daty type
         """
 
-        base = HealpixBase.moc_from_pixels(pixels, order, nest = nest)
+        base = HealpixBase.moc_from_pixels(nside, pixels, nest = nest)
 
         data = np.rec.fromarrays([base._uniq, np.zeros(base.npix, dtype = dtype)],
                                  names = ["UNIQ", "CONTENTS"])
@@ -499,20 +506,18 @@ class HealpixMap(HealpixBase):
 
             return value
                 
-        order = hp.nside2order(nside)
-
         if weights:
             dtype = array(weights).dtype
         else:
             dtype = int
         
-        moc_map = cls.adaptive_moc_mesh(lambda i,f: value_fun(i,f) > max_value,
-                                        order,
+        moc_map = cls.adaptive_moc_mesh(nside,
+                                        lambda i,f: value_fun(i,f) > max_value,
                                         density = False,
                                         dtype = dtype)
 
         # Fill
-        rangesets = moc_map.pix_rangesets(order)
+        rangesets = moc_map.pix_rangesets(nside)
 
         for pix,(start,stop) in enumerate(rangesets):
             
@@ -528,10 +533,13 @@ class HealpixMap(HealpixBase):
 
         ... note:: 
             
-            The maximum order of the MOC map is the same as the order of the 
+            The maximum nside of the MOC map is the same as the nside of the 
             single-resolution map, so the output map could contain pixels with 
             a value greater than this.
 
+        If the map is already a MOC map, it will recompute the grid accordingly
+        by combining uniq pixels. Uniq pixels are never split. 
+
         Also see the more generic ``adaptive_moc_map()`` and ``adaptive_moc_mesh()``.
 
         Args:
@@ -542,32 +550,77 @@ class HealpixMap(HealpixBase):
             HealpixMap
         """
 
-        if self.is_moc:
-            return TypeError("Map is already a MOC")
-        
         # Get empty mesh by reusing adaptive_moc_mesh
-        def value_fun(start, stop):
-            value = sum(self._data[start:stop])
+        if self.is_moc:
+            # MOC map, will work in rangesets
+            
+            rs = self.pix_rangesets(self.nside)
+            rs_argsort = np.argsort(rs.start)
 
-            if self._density:
-                value /= stop-start
+            def _nest2pix(start,stop):
+                # Same as nest2pix but avoids recomputing the rangesets and sorting
+                
+                return  np.searchsorted(rs.start, [start,stop],
+                                        sorter = rs_argsort)
 
-            return value
+                
+            def value_fun(start, stop):
 
-        if self._density:
-            def split_fun(start, stop):
-                return max(self._data[start:stop]) > max_value
-        else:
+                start_pos, stop_pos = _nest2pix(start, stop)
+
+                pix = rs_argsort[start_pos:stop_pos]
+
+                if self._density:
+                    # Weighted average
+                    value = sum((rs.stop[pix]-rs.start[pix])*self._data[pix])
+                    value /= rs.stop[stop_pos-1] - rs.start[start_pos]
+                else:
+                    # Simple sum
+                    value = sum(self._data[pix])
+
+                return value
+                    
             def split_fun(start,stop):
-                return value_fun(start,stop) > max_value
+
+                start_pos, stop_pos = _nest2pix(start, stop)
+
+                if stop_pos - start_pos == 1:
+                    # A single pixel, don't split
+                    return False
+                else:
+
+                    pix = rs_argsort[start_pos:stop_pos]
+                    
+                    if self._density:
+                        return max(self._data[pix]) > max_value
+
+                    else:
+                        return sum(self._data[pix]) > max_value
+                    
+        else:
+            # Single resolution map
+            
+            def value_fun(start, stop):
+                value = sum(self._data[start:stop])
+
+                if self._density:
+                    value /= stop-start
+
+                return value
+
+            def split_fun(start, stop):
+                if self._density:
+                    return max(self._data[start:stop]) > max_value
+                else:
+                    return value_fun(start,stop) > max_value
         
-        moc_map = self.adaptive_moc_mesh(split_fun,
-                                         self.order,
+        moc_map = self.adaptive_moc_mesh(self.nside,
+                                         split_fun,
                                          density = self._density,
                                          dtype = self.dtype)
 
         # Fill
-        rangesets = moc_map.pix_rangesets(self.order)
+        rangesets = moc_map.pix_rangesets(self.nside)
 
         for pix,(start,stop) in enumerate(rangesets):
             
@@ -625,8 +678,8 @@ class HealpixMap(HealpixBase):
         
     def __eq__(self, other):
 
-        return (super().__eq__(self, other) and
-                array_equal(self._data, other._data) and
+        return (self.conformable(other) and
+                np.array_equal(self._data, other._data) and
                 self._density == other._density)
                 
     def __getitem__(self, key):
@@ -646,8 +699,8 @@ class HealpixMap(HealpixBase):
         return self._operation(other, operator.mul)
 
     def __rmul__(self, other):
-            
-        return self._operation(other, operator.mul)
+
+        return self._roperation(other, operator.mul)
 
     def __itruediv__(self, other):
 
@@ -658,8 +711,8 @@ class HealpixMap(HealpixBase):
         return self._operation(other, operator.truediv)
 
     def __rtruediv__(self, other):
-            
-        return self._operation(other, operator.truediv)
+
+        return self._roperation(other, operator.truediv)
 
     def __ifloordiv__(self, other):
 
@@ -670,8 +723,8 @@ class HealpixMap(HealpixBase):
         return self._operation(other, operator.floordiv)
 
     def __rfloordiv__(self, other):
-            
-        return self._operation(other, operator.floordiv)
+
+        return self._roperation(other, operator.floordiv)
 
     def __iadd__(self, other):
 
@@ -683,7 +736,7 @@ class HealpixMap(HealpixBase):
 
     def __radd__(self, other):
             
-        return self._operation(other, operator.add)
+        return self._roperation(other, operator.add)
 
     def __isub__(self, other):
 
@@ -694,8 +747,8 @@ class HealpixMap(HealpixBase):
         return self._operation(other, operator.sub)
 
     def __rsub__(self, other):
-            
-        return self._operation(other, operator.sub)
+ 
+        return self._roperation(other, operator.sub)
 
     def __ipow__(self, other):
 
@@ -705,10 +758,6 @@ class HealpixMap(HealpixBase):
 
         return self._operation(other, operator.pow)
 
-    def __rpow__(self, other):
-            
-        return self._operation(other, operator.pow)
-
     def __neg__(self):
 
         new = deepcopy(self)
@@ -761,6 +810,18 @@ class HealpixMap(HealpixBase):
             raster._ioperation(self, lambda a,b: b)
 
             return raster
+
+    def _roperation(self, other, operation):
+
+        if not np.isscalar(other):
+            raise ValueError("Operations can only occur between maps or a "
+                             "map and a scalar")
+
+        m = deepcopy(self)
+
+        m._data = operation(other, m._data)
+
+        return m
         
     def _operation(self, map1, operation):
 
@@ -781,11 +842,12 @@ class HealpixMap(HealpixBase):
 
         else:
 
-            # Operation by another map or something that can be turned into a map
+            # Operation by another map
 
             # Cast to HealpixMap if needed
             if not isinstance(map1, HealpixMap):
-                map1 = HealpixMap(map1)
+                raise ValueError("Operations can only occur between maps or a "
+                                 "map and a scalar")
 
             # Optimize for different cases
             if (map0.is_moc or map1.is_moc) and not map0.conformable(map1):
@@ -795,10 +857,10 @@ class HealpixMap(HealpixBase):
 
                 # Convert pixel numbers to an equivalent sorted list of nested
                 # rangeset for highest posible order
-                max_order = max(map0.order, map1.order)
+                max_nside = max(map0.nside, map1.nside)
 
-                rs0 = map0.pix_rangesets(max_order)
-                rs1 = map1.pix_rangesets(max_order)
+                rs0 = map0.pix_rangesets(max_nside)
+                rs1 = map1.pix_rangesets(max_nside)
 
                 sort0 = np.argsort(rs0.start)
                 sort1 = np.argsort(rs1.start)
@@ -842,8 +904,8 @@ class HealpixMap(HealpixBase):
                     # Operation
                     value = operation(value0, value1)
 
-                    new_uniq[pix_new] = hmap.range2uniq(max_order,
-                                                              (start, stop))
+                    new_uniq[pix_new] = hmap.range2uniq(max_nside,
+                                                        (start, stop))
                     new_data[pix_new] = value
 
                     # Advance to next pixel
@@ -895,7 +957,8 @@ class HealpixMap(HealpixBase):
 
             # Cast to HealpixMap if needed
             if not isinstance(map1, HealpixMap):
-                map1 = HealpixMap(map1)
+                raise ValueError("Operations can only occur between maps or a "
+                                 "map and a scalar")
 
             # Optimize procedure for various situations
             if map0.conformable(map1):
@@ -909,13 +972,13 @@ class HealpixMap(HealpixBase):
                 # There is no clear optimization in this case
                 # Will work in the rangeset representation
 
-                max_order = max(map0.order, map1.order)
+                max_nside = max(map0.nside, map1.nside)
 
-                rs0   = map0.pix_rangesets(max_order)
+                rs0   = map0.pix_rangesets(max_nside)
                 sort0 = np.argsort(rs0.start)
                 pos0  = 0
 
-                rs1   = map1.pix_rangesets(max_order)
+                rs1   = map1.pix_rangesets(max_nside)
                 sort1 = np.argsort(rs1.start)
                 pos1  = 0
 
@@ -1118,39 +1181,9 @@ class HealpixMap(HealpixBase):
             ax.axis('off')
             default_plot = True
             
-        # Setup appropiate projection
-        proj = proj.lower()
-        
-        if proj == 'moll':            
-            projector = hp.projector.MollweideProj(rot=rot,
-                                              coord=coord,
-                                              flipconv=flip,
-                                              xsize=xsize)
-            
-        elif proj == 'cart':
-            projector = hp.projector.CartesianProj(rot=rot,
-                                              coord=coord,
-                                              flipconv=flip,
-                                              xsize=xsize,
-                                              ysize=ysize,
-                                              lonra=lonra,
-                                              latra=latra)
-        elif proj == 'orth':
-            projector = hp.projector.OrthographicProj(rot=rot,
-                                                 coord=coord,
-                                                 flipconv=flip,
-                                                 xsize=xsize,
-                                                 half_sky=half_sky)
-        elif proj == 'gnom':
-            projector = hp.projector.GnomonicProj(rot=rot,
-                                             coord=coord,
-                                             flipconv=flip,
-                                             xsize=xsize,
-                                             ysize=ysize,
-                                             reso=reso)
-            
-        else:
-            raise ValueError("Wrong porojection")
+        # Get appropiate projector
+        projector = self._get_projector(proj, rot, coord, flip, xsize, ysize,
+                                        lonra, latra, half_sky, reso)
         
         # Get values
         class rasterizer:
@@ -1169,8 +1202,10 @@ class HealpixMap(HealpixBase):
                 if self.map.is_moc and not self.map._density:
 
                     # Histogram-like MOC, will divide the pixel
-                    pix_order = hmap.uniq2order(self.map._uniq[pix]) 
+                    pix_nside = hmap.uniq2nside(self.map._uniq[pix]) 
 
+                    pix_order = log2(pix_nside)
+                    
                     npix_ratio = 4 ** (self.map.order - pix_order)
 
                     return  self.map[pix] / npix_ratio
@@ -1216,14 +1251,14 @@ class HealpixMap(HealpixBase):
 
             values = self[pixels]
             
-            if not self.density:
+            if not self._density:
                 # Split the pixels up to corresponding maximum order
                 
-                order = hmap.uniq2order(self._uniq[pixels])
+                nside = hmap.uniq2nside(self._uniq[pixels])
 
-                npix_ratio = 4 ** (self.order - order)
-
-                values /= npix_ratio
+                npix_ratio =  int(4**self.order) / nside / nside
+                
+                values = values / npix_ratio
 
             return sum(values * weights)
             
@@ -1232,4 +1267,19 @@ class HealpixMap(HealpixBase):
             return hp.get_interp_val(self._data, theta, phi, nest = self.is_nested)
 
 
-        
+    def moc_sort(self):
+        """
+        Sort the uniq pixels composing a MOC map based on its 
+        rangeset representation
+        """
+
+        if not self.is_moc:
+            return
+
+        rs = self.pix_rangesets(self.nside)
+
+        rs_argsort = np.argsort(rs.start)
+
+        self._uniq = self._uniq[rs_argsort]
+        self._data = self._data[rs_argsort]
+
